#!/usr/bin/python -u

"""
Allocates resources from Murphy and/or a make jobserver while running
some command.
"""

import time
import os
import re
import subprocess
import signal
from optparse import OptionParser

usage = "usage: %prog [options] [--] command arg1 arg2 ..."
parser = OptionParser(usage=usage)
parser.add_option("-r", "--murphy-resource",
                  dest="resources",
                  action="append",
                  help="Name of a Muprhy resource which gets locked while running the command.")
parser.add_option("-j", "--jobs",
                  default=1,
                  type='int',
                  action="store",
                  help="Number of jobs to allocate from job server. Ignored if not running under a job server.")

(options, args) = parser.parse_args()

def log(format, *args):
    now = time.time()
    print time.asctime(time.gmtime(now)), 'UTC', '(+ %.1fs / %.1fs)' % (now - log.latest, now - log.start), format % args
    log.latest = now
log.start = time.time()
log.latest = log.start

# Murphy support: as a first step, lock one resource named like the
# test before running the test.
gobject = None
if options.resources:
    try:
        import gobject
    except ImportError:
        from gi.repository import GObject as gobject
    import dbus
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)
    if not os.environ.get('DBUS_SESSION_BUS_ADDRESS', None):
        # Try to set up Murphy with a murphy-launch.py helper script
        # which is expected to be provided by the test environment
        # (not included in SyncEvolution).
        vars = subprocess.check_output(['murphy-launch.py'])
        for line in vars.split('\n'):
            if line:
                var, value = line.split('=', 1)
                os.environ[var] = value
    bus = dbus.SessionBus()
    loop = gobject.MainLoop()
    murphy = dbus.Interface(bus.get_object('org.Murphy', '/org/murphy/resource'), 'org.murphy.manager')

    # Support mapping of resource "foo" to "bar" with RESOURCES_FOO=bar.
    resources = []
    for name in options.resources:
        replacement = os.environ.get('RESOURCES_%s' % name.upper(), None)
        if replacement is not None:
            resources.extend(replacement.split(','))
        else:
            resources.append(name)

    if resources != options.resources:
        log('replaced resource set %s with %s based on RESOURCES_* env vars', options.resources, resources)

    if resources:
        log('=== locking resource(s) %s ===', resources)
        resourcesetpath = murphy.createResourceSet()
        resourceset = dbus.Interface(bus.get_object('org.Murphy', resourcesetpath), 'org.murphy.resourceset')
        for name in resources:
            resourcepath = resourceset.addResource(name)
            # Allow sharing of the resource. Only works if the resource
            # was marked as "shareable" in the murphy config, otherwise
            # we get exclusive access.
            resource = dbus.Interface(bus.get_object('org.Murphy', resourcepath), 'org.murphy.resource')
            resource.setProperty('shared', dbus.Boolean(True, variant_level=1))

        # Track pending request separately, because status == 'pending'
        # either means something else ('unknown'?) or is buggy/unreliable.
        # See https://github.com/01org/murphy/issues/5
        pending = False
        def propertyChanged(prop, value):
            global pending
            log('property changed: %s = %s', prop, value)
            if prop == 'status':
                if value == 'acquired':
                    # Success!
                    loop.quit()
                elif value == 'lost':
                    # Not yet?!
                    log('Murphy request failed, waiting for resource to become available.')
                    pending = False
                elif value == 'pending':
                    pass
                elif value == 'available':
                    if not pending:
                        log('Murphy request may succeed now, try again.')
                        resourceset.request()
                        pending = True
                    else:
                        log('Unexpected status: %s', value)
        try:
            match = bus.add_signal_receiver(propertyChanged, 'propertyChanged', 'org.murphy.resourceset', 'org.Murphy', resourcesetpath)
            resourceset.request()
            pending = True
            loop.run()
        finally:
            match.remove()

class Jobserver:
    '''Allocates the given number of job slots from the "make -j"
    jobserver, then runs the command and finally returns the slots.
    See http://mad-scientist.net/make/jobserver.html'''
    def __init__(self):
        self.havejobserver = False
        self.allocated = 0

        # MAKEFLAGS= --jobserver-fds=3,4 -j
        flags = os.environ.get('MAKEFLAGS', '')
        m = re.search(r'--jobserver-fds=(\d+),(\d+)', flags)
        if m:
            self.receiveslots = int(m.group(1))
            self.returnslots = int(m.group(2))
            self.blocked = {}
            self.havejobserver = True
            log('using jobserver')
        else:
            log('not using jobserver')

    def active(self):
        return self.havejobserver

    def alloc(self, numjobs = 1):
        if not self.havejobserver:
            return
        n = 0
        self._block()
        try:
            while n < numjobs:
                os.read(self.receiveslots, 1)
                n += 1
            self.allocated += n
            n = 0
        except:
            os.write(self.returnslots, ' ' * n)
            raise
        finally:
            self._unblock()

    def free(self, numjobs = 1):
        if not self.havejobserver:
            return
        try:
            self.allocated -= numjobs
            os.write(self.returnslots, ' ' * numjobs)
        finally:
            self._unblock()

    def _block(self):
        '''Block signals if not already done.'''
        if not self.blocked:
            for sig in [ signal.SIGINT, signal.SIGTERM ]:
                self.blocked[sig] = signal.signal(sig, signal.SIG_IGN)

    def _unblock(self):
        '''Unblock signals if blocked and we currently own no slots.'''
        if self.blocked and not self.allocated:
            for sig, handler in self.blocked.items():
                signal.signal(sig, handler)
            self.blocked = {}

jobserver = Jobserver()

jobs = 0
if jobserver.active() and options.jobs:
    log('=== allocating %d job slot(s) ===', options.jobs)
    jobserver.alloc(options.jobs)
    log('=== allocated %d job slot(s) ===', options.jobs)
    jobs = options.jobs

try:
    subprocess.check_call(args)
finally:
    log('=== cleaning up ===')
    # Return job tokens.
    if jobs:
        jobserver.free(jobs)
    # We don't need to unlock the Murphy resource. Quitting will do
    # that automatically.



from twisted.internet import reactor
from twisted.web import http
from twisted.web.proxy import Proxy, ProxyRequest, ProxyClient, ProxyClientFactory
from twisted.python import log
import sys

class ContentWrapper:
    def __init__(self, content, parent):
        self.content = content
        self.parent = parent

    def seek(self, a, b):
        self.content.seek(a,b)

    def read(self):
        s = self.content.read()
        if True:
            log.msg("interrupt before send")
            # This exception will abort contacting the server and appear as
            # "Unhandled Error" in the proxy log. The proxy client will get
            # an empty reply.
            raise EOFError()
        return s

    def close(self):
        self.content.close()

class MyProxyClient(ProxyClient):
    def __init__(self, command, rest, version, headers, data, father):
        ProxyClient.__init__(self, command, rest, version, headers, data, father)

    def connectionMade(self):
        ProxyClient.connectionMade(self)
        log.msg("message sent")
        # interrupt now before server can reply?
        if self.father.mode == MyProxyRequest.INTERRUPT_AFTER_SEND:
            log.msg("interrupt after sending")
            # finish writing, but never read
            self.transport.loseConnection()
            # Be nice and report a real error back to the proxy client.
            self.father.setResponseCode(501, "Gateway error")
            self.father.responseHeaders.addRawHeader("Content-Type", "text/plain")
            self.father.write("connection intentionally interrupted after sending and before receiving")

class MyProxyClientFactory(ProxyClientFactory):
    protocol = MyProxyClient

    def __init__(self, command, rest, version, headers, data, father):
        ProxyClientFactory.__init__(self, command, rest, version, headers, data, father)

class MyProxyRequest(ProxyRequest):
    protocols = {"http": MyProxyClientFactory}
    INTERRUPT_BEFORE_SEND = 1
    INTERRUPT_AFTER_SEND = 2
    INTERRUPT_AFTER_RECEIVE = 3
    baseport = 10000

    def __init__(self, channel, queued, reactor=reactor):
        ProxyRequest.__init__(self, channel, queued, reactor)
        self.mode = channel.transport.server.port - self.baseport

    def process(self):
        log.msg("mode is", self.mode)

        # override read() method so that we can influence the original
        # process() without having to copy it; just replacing
        # the read method inside the existing content instance
        # would be easier, but turned out to be impossible (read-only
        # attribute)
        if self.mode == self.INTERRUPT_BEFORE_SEND:
            # ContentWrapper will raise exception instead of delivering data
            self.content = ContentWrapper(self.content, self)
        ProxyRequest.process(self)

    def write(self, content):
        log.msg("reply:", content)
        if self.mode == self.INTERRUPT_AFTER_RECEIVE:
            # TODO: suppress original headers
            # Original headers already sent to proxy client, but we
            # can still suppress the actual data and close the
            # connection to simulate a failure.
            log.msg("interrupt after receive")
            ProxyRequest.write(self, "")
            self.transport.loseConnection()
        else:
            ProxyRequest.write(self, content)

class MyProxy(Proxy):
    requestFactory = MyProxyRequest

if __name__ == '__main__':
    log.startLogging(sys.stdout)

    f = http.HTTPFactory()
    f.protocol = MyProxy
    reactor.listenTCP(MyProxyRequest.baseport + 0, f)
    reactor.listenTCP(MyProxyRequest.baseport + MyProxyRequest.INTERRUPT_BEFORE_SEND, f)
    reactor.listenTCP(MyProxyRequest.baseport + MyProxyRequest.INTERRUPT_AFTER_SEND, f)
    reactor.listenTCP(MyProxyRequest.baseport + MyProxyRequest.INTERRUPT_AFTER_RECEIVE, f)

    reactor.run()




#!/usr/bin/python
#
# Copyright (C) 2010 Intel Corporation
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301  USA

'''
Automatically trying different configurations for a phone to sync with
SyncEvolution.
'''
import sys, optparse, os, time, tempfile
import shutil
import ConfigParser
import glob
import os.path

# source names as commonly used in SyncEvolution
allSources = ['addressbook', 'calendar', 'todo', 'memo', 'calendar+todo']
# valid SyncMLVersion values
allVersions = ['1.2', '1.1', '1.0']

########################### cmdline options ##########################################
parser = optparse.OptionParser()
parser.add_option("-b", "--bt-address", action = "store", type = "string",
        dest = "btaddr", help = "The Bluetooth mac address for the testing phone", default = "")
parser.add_option ("-p", "--protocol-version", action = "store", type = "string",
                   dest="version", help = "The SyncML protocal version for testing, can be one of " +
                   "|".join(allVersions) + ", by default it will try all versions one by one",
                   default = "")
parser.add_option ("-s", "--source", action = "store", type = "string", dest=
        "source", help = "The local database for testing, can be one of " + "|".join(allSources),
        default = "")
parser.add_option ("-u", "--uri", action = "store", type = "string", dest =
        "uri", help = "The URI for testing the selected source, invalid when no specific source is selected via --source", default = "")
parser.add_option ("-t", "--type", action = "store", type = "string", dest =
        "type", help = "The content type for testing the selected source, invalid when no specific source is selected via --source"
        ,default = "")
parser.add_option ("-i", "--identifier", action = "store", type="string",
        dest = "identifier", help = "The identifier used when contacting the phone, can be arbitray string. By default it will try 'PC Suite','Nokia PC Suite' and empty string",
        default = "")
parser.add_option ("", "--without-ctcap", action = "store_true", default =False,
        dest = "ctcap", help = "Testing without sending CTCap information")
parser.add_option ("-v", "--verbose", action = "count",
        dest = "verbose", help = "Increase amount of output")
parser.add_option ("-a", "--advanced", action = "store_true", default = False,
        dest = "advanced", help = "More extensive test with sending/receving data, WARNING: will destroy your data on the tested phone")
parser.add_option("-c", "--create-config", action ="store", type = "string", 
        dest = "create", help = "If set, a configuration file with the name will be created based on the testing result",
        default ="")
(options, args) = parser.parse_args()

####################semantic check for  cmdline options #######################################
if not options.btaddr:
    parser.error ("Please provide the Bluetooth MAC address for the phone with -b/--bt-address.")
if options.version and options.version not in allVersions:
    parser.error("Option -p/--protocol-version can only be one of " + "|".join(allVersions) + ".")
if options.source and options.source not in allSources:
    parser.error("Option -s/--source can only be one of " + "|".join(allSources) + ".")
if options.uri and not options.source:
    parser.error ("Option -u/--uri only works in combination with -s/--source.")
if options.type and not options.source:
    parser.error ("Option -t/--type only works in combination with -s/--source.")

#######################some global parameters ######################
syncevoCmd = 'syncevolution'
configName = 'test-phone'        # inside temporary testConfig dir
# real paths set in main() inside temporary directory
testFolder = '/dev/null/data'
testResult = '/dev/null/cache'
testConfig = '/dev/null/config'


#################### Configuration Parameter #######################
class ConfigurationParameter:
    def __init__ (self, version, source, uri, type, ctcap, identifier):
        self.version = version 
        self.source = source
        self.uri = uri 
        self.type = type 
        self.ctcap = ctcap 
        self.identifier = identifier

    def printMe(self):
        print "Test parameter: "
        print "With CTCap:     %s" %(self.ctcap,)
        print "Identifier:     %s" %(self.identifier,)
        print "SyncML version: %s" %(self.version,)
        print "Sync Source:    %s" %(self.source,)
        print "URI:            %s" %(self.uri,)
        print "Content Type:   %s" %(self.type,)

    def __str__(self):
        res = []
        if self.ctcap:
            res.append("with CTCap")
        else:
            res.append("without CTCap")
        res.append(self.identifier)
        res.append(self.version)
        res.append("%s = %s + %s" % (self.source, self.uri, self.type))
        return ", ".join(res)

    def equalWith(self, config):
        return (config and \
                self.ctcap == config.ctcap and \
                self.identifier == config.identifier and \
                self.version == config.version and \
                self.uri == config.uri and \
                self.type == config.type)

###################### utility functions ####################
def clearLocalSyncData(sources):
    for source in sources:
        dirname = "%s/%s" % (testFolder, source)
        rm_r(dirname)
        os.makedirs(dirname)

def createFile(filename, content):
    f = open(filename, "w")
    f.write(content)

def insertLocalSyncData(sources, type):
    for source in sources:
        testcase, keys = getTestCase (source, type)
        createFile(os.path.join(testFolder, source, "0"), testcase)

def getTestCase(source, type):
    """Returns a pair of test item string plus a list of sub strings
    which are expected to come back from the phone. Type comparison is
    intentionally a bit vague, so that it doesn't matter whether the
    type contains a version or a ! force flag."""
    if source == 'addressbook' and type.startswith('text/vcard'):
        return  ("BEGIN:VCARD\n"
                 "VERSION:3.0\n"
                 "TITLE:tester\n"
                 "FN:John Doe\n"
                 "N:Doe;John;;;\n"
                 "TEL;TYPE=WORK;TYPE=VOICE:business 1\n"
                 "X-EVOLUTION-FILE-AS:Doe\\, John\n"
                 "X-MOZILLA-HTML:FALSE\n"
                 "NOTE:test-phone\n"
                 "END:VCARD\n",
                 ["VCARD", "TITLE:tester", "Doe", "John"])

    if source == 'addressbook' and type.startswith('text/x-vcard'):
        return ("BEGIN:VCARD\n"
                "VERSION:2.1\n"
                "TITLE:tester\n"
                "FN:John Doe\n"
                "N:Doe;John;;;\n"
                "TEL;TYPE=WORK;TYPE=VOICE:business 1\n"
                "X-MOZILLA-HTML:FALSE\n"
                "NOTE:REVISION\n"
                "END:VCARD\n",
                ["VCARD", "TITLE:tester", "Doe", "John"])

    if source == 'calendar' and type.startswith('text/calendar'):
        return ("BEGIN:VCALENDAR\n"
                "PRODID:-//Ximian//NONSGML Evolution Calendar//EN\n"
                "VERSION:2.0\n"
                "METHOD:PUBLISH\n"
                "BEGIN:VEVENT\n"
                "SUMMARY:phone meeting\n"
                "DTEND:20060406T163000Z\n"
                "DTSTART:20060406T160000Z\n"
                "DTSTAMP:20060406T211449Z\n"
                "LAST-MODIFIED:20060409T213201\n"
                "CREATED:20060409T213201\n"
                "LOCATION:my office\n"
                "DESCRIPTION:let's talkREVISION\n"
                "END:VEVENT\n"
                "END:VCALENDAR\n",
                ["VCALENDAR", "VEVENT", "phone meeting", "my office"])

    if source == 'calendar' and type.startswith('text/x-vcalendar'):
        return ("BEGIN:VCALENDAR\n"
                "VERSION:1.0\n"
                "BEGIN:VEVENT\n"
                "SUMMARY:phone meeting\n"
                "DTEND:20060406T163000Z\n"
                "DTSTART:20060406T160000Z\n"
                "DTSTAMP:20060406T211449Z\n"
                "LOCATION:my office\n"
                "DESCRIPTION:let's talkREVISION\n"
                "END:VEVENT\n"
                "END:VCALENDAR\n",
               ["VCALENDAR", "VEVENT", "phone meeting", "my office"])
 
    if source == 'todo' and type.startswith('text/calendar'):
        return ("BEGIN:VCALENDAR\n"
                "PRODID:-//Ximian//NONSGML Evolution Calendar//EN\n"
                "VERSION:2.0\n"
                "METHOD:PUBLISH\n"
                "BEGIN:VTODO\n"
                "DTSTAMP:20060417T173712Z\n"
                "SUMMARY:do me\n"
                "DESCRIPTION:to be doneREVISION\n"
                "CREATED:20060417T173712\n"
                "LAST-MODIFIED:20060417T173712\n"
                "END:VTODO\n"
                "END:VCALENDAR\n",
                ["VCALENDAR", "VTODO", "do me"]) 

    if source == 'todo' and type.startswith('text/x-vcalendar'):
        return ("BEGIN:VCALENDAR\n"
                "PRODID:-//Ximian//NONSGML Evolution Calendar//EN\n"
                "VERSION:1.0\n"
                "METHOD:PUBLISH\n"
                "BEGIN:VTODO\n"
                "DTSTAMP:20060417T173712Z\n"
                "SUMMARY:do me\n"
                "DESCRIPTION:to be doneREVISION\n"
                "CREATED:20060417T173712\n"
                "LAST-MODIFIED:20060417T173712\n"
                "END:VTODO\n"
                "END:VCALENDAR\n",
                ["VCALENDAR", "VTODO", "do me"])

    if source == 'memo':
        return ("Summary Line\n"
                "BODY TEXT\n",
                ["Summary Line"])

    raise "no test data defined for source %s and type %s" % (source, type)

# Compare the received data with the sent data, we only match selected keywords in received
# data for a basic sanity test
def compareSyncData(sources, type):
    for source in sources:
        testcase, keys  = getTestCase (source, type)
        received = ''
        recFile = "%s/%s/0" %(testFolder, source)
        try:
            rf = open (recFile)
            received = rf.read()
        except:
            return False
        if (options.verbose > 1):
            print "comparing received file:"
            print received
            print "with built in keywords in test case:"
            print keys
        for key in keys:
            if (received.find(key) <0):
                return False
        return True

# wrapper of running a shell command
def runCommand(cmd, exception = True):
    """Log and run the given command, throwing an exception if it fails."""
    if (options.verbose > 1):
        print "%s: %s" % (os.getcwd(), cmd)
    else:
        cmd += ' >/dev/null'
    sys.stdout.flush()
    result = os.system(cmd)
    if result != 0 and exception:
        raise Exception("%s: failed (return code %d)" % (cmd, result>>8))

def runSync(sync):
    rm_r("%s/syncevolution" % testResult)
    status = True
    interrupt = False

    try:
        runCommand (sync)
    except:
        status = False
        pass

    # session name is unknown, but we know there is only one, so let
    # glob find it for us
    resultFile = glob.glob("%s/syncevolution/*/status.ini"  % testResult)[0]

    # inject [main] at start of file for ConfigParser,
    # because SyncEvolution doesn't write sections
    class IniFile:
        def __init__(self, filename):
            self.fp = open(filename, "r")
            self.read = False
        def readline(self):
            if not self.read:
                self.read = True
                return "[main]"
            else:
                return self.fp.readline()

    ini = ConfigParser.ConfigParser({"status": "0", "error": ""})
    ini.readfp(IniFile(resultFile))
    statuscode = ini.get("main", "status")
    if statuscode == "20015":
        # aborted by user, stop testing
        status = False
        interrupt = True
    if statuscode == "22002":
        # syncevolution failed (for example, kill -9), warn and abort
        print "\nSyncEvolution binary died prematurely, aborting testing."
        status = False
        interrupt = True
    return (status, interrupt)

# recursive directory removal, without throwing an error if directory does not exist
def rm_r(dirname):
    if os.path.isdir(dirname):
        shutil.rmtree(dirname)

def hash2ini(hash):
    """convert key/value pairs into .ini file without sections"""
    res = []
    for key, value in hash.items():
        res.append("%s = %s" % (key, value))
    return "\n".join(res)

def strip_version(type):
    """turn type[:version][!] into type[!]"""
    res = type.split(':')[0]
    if type.endswith('!'):
        res += '!'
    return res

##############################TestConfiguration##################################
class TestingConfiguration():
    def __init__(self, versions, sources, uris, types, ctcaps, identifiers, btaddr):
        if (versions):
            self.versions = versions
        else:
            self.versions = allVersions

        # If "calendar+todo" is tested, then "calendar" and "todo"
        # must be tested first. If they lead to the same result,
        # then they have to be combined for "calendar+todo".
        # "calendar+todo" itself is never tested directly.
        if sources:
            self.sources = sources
        else:
            self.sources = allSources
        if "calendar+todo" in self.sources:
            self.sources.remove("calendar+todo")
            if not "calendar" in self.sources:
                self.sources.append("calendar")
            if not "todo" in self.sources:
                self.sources.append("todo")

        if (uris):
            self.uris = uris
        else:
            self.uris = {}
            self.uris['addressbook'] = ['Contact', 'contact', 'Contacts', 'contacts', 'Addressbook', 'addressbook']
            self.uris['calendar'] = ['Calendar', 'calendar', 'Agenda','agenda']
            self.uris['todo'] = self.uris['calendar'] + ['Task', 'task', 'Tasks', 'tasks', 'Todo','todo']
            self.uris['memo'] = ['Memo', 'memo', 'Notes', 'notes', 'Note', 'note']

        if (types):
            self.types = types
        else:
            # - must include version numbers because file backend needs them
            # - current types like 'text/vcard:3.0' are "downgraded" to the
            #   legacy types when sending a SAN, so they are basically identical;
            #   to really send a SAN with these current types, we have to "force" them
            self.types = {}
            self.types['addressbook'] = ['text/vcard:3.0', 'text/x-vcard:2.1', 'text/vcard:3.0!']
            self.types['calendar'] = self.types['todo'] = ['text/calendar:2.0', 'text/x-vcalendar:1.0', 'text/calendar:2.0!']
            self.types['memo'] = ['text/plain:1.0']

        if (ctcaps):
            self.ctcaps = ctcaps
        else:
            self.ctcaps =  [True, False]
        if (identifiers):
            self.identifiers = identifiers
        else:
            self.identifiers = ['PC Suite','','Nokia PC Suite']
        self.btaddr = btaddr


    #before each configuration is really tested, prepare is called.
    #returns True if we decide current configuration need not be tested
    def prepare (self, allconfigs, curconfig):
        # Decide whether this config should be skipped (because we already found
        # a working configuration
        # Test is skipped either because 
        # 1) we already found a working configuration for the data source;
        # 2) based on the working configuration for source A, we can reasonably
        # guess a working configuration for source B must have the same
        # 'identifier', 'ctcap' and 'SyncMLVersion' setting.
        # 3) we already found a working configuration for combined calendar and
        # task, thus seperate testing for calendar and task is not needed.
        skip = False
        for source, config in self.wConfigs.items():
            if (config):
                if ( (config.source == self.source) or (config.identifier != self.identifier ) or (config.ctcap != self.ctcap) or (config.version != self.version)):
                    skip = True
        if (skip):
            if (options.verbose > 1):
                print "Test %d/%d skipped because already found a working configuration" % (curconfig, allconfigs)
            elif options.verbose > 0:
                print "Test %d/%d skipped" %(curconfig, allconfigs), \
                    ConfigurationParameter(self.version, self.source, self.uri, self.type, self.ctcap, self.identifier)
            else:
                print "Test %d/%d skipped" %(curconfig, allconfigs)
        else:
            print ("Start %d/%d test" % (curconfig, allconfigs)),
            if (options.verbose > 0):
                config = ConfigurationParameter(self.version, self.source, self.uri, self.type, self.ctcap, self.identifier)
                if (options.verbose > 1):
                    print
                    config.printMe()
                else:
                    print config
            else:
                print

        return skip


    #run the real sync test with current configuration parameter
    #if advanced option is set and the basic test succeed, it will contintue with 
    #the advanced test
    def testWithCurrentConfiguration(self):
        """ Prepare the configuration and run a sync session, Returns true if
        the test was successful, otherwise false"""
        rm_r(testConfig)
        cmdPrefix = "XDG_CACHE_HOME=%s XDG_CONFIG_HOME=%s " %(testResult, testConfig)
        syncevoTest = "%s %s --daemon=no" % (cmdPrefix, syncevoCmd)
        runCommand ("%s -c --template 'SyncEvolution Client' --sync-property peerIsClient=1 %s" % (syncevoTest, configName))
        # set the local database
        filesource = testFolder+'/'+self.source
        configCmd = "%s --configure --source-property evolutionsource='file:///%s' %s %s" %(syncevoTest, filesource, configName, self.source) 
        runCommand (configCmd)

        configCmd = "%s --configure --sync-property logLevel=5 --sync-property SyncURL=obex-bt://%s --sync-property SyncMLVersion=%s %s" % (syncevoTest, self.btaddr,self.version, configName)
        runCommand (configCmd)

        if (self.identifier):
            configCmd = "%s --configure --sync-property remoteIdentifier='%s' %s" %(syncevoTest, self.identifier, configName)
            runCommand (configCmd)

        configCmd = "%s --configure --source-property 'type=file:%s' --source-property uri=%s %s %s" %(syncevoTest, self.type, self.uri, configName, self.source)
        runCommand (configCmd)

        """ start the sync session """
        if (not self.ctcap):
            cmdPrefix += "SYNCEVOLUTION_NOCTCAP=t "
        cmdPrefix += "SYNCEVOLUTION_NO_SYNC_SIGNALS=1"
        syncCmd = " ".join((cmdPrefix, syncevoCmd, "--daemon=no", configName, self.source))
        (status,interrupt) = runSync(syncCmd)
        if (options.advanced and status and not interrupt):
            (status,interrupt)= self.advancedTestWithCurrentConfiguration()
        return (status, interrupt)

    '''Basic test for sending/receiving data
    It will work as:
    Clear local data and data on the phone via 'slow-sync' and 'two-way' sync
    Send local test case to the phone via 'two-way'
    Clear local data and get the data from the phone via 'slow-sync'
    compare the sent data with the received data to decide whether the test was successful

    Note that this depends on the phone support 'slow-sync' and 'two-way' sync and 
    implements the semantics correctly as specified in the spec. Otherwise the results will
    be undefined.
    '''
    def advancedTestWithCurrentConfiguration (self):
        """ 
        Sending/receving real data for basic sanity test
        """
        sources = []
        sources.append(self.source)

        #step 1: clean the data both locally and remotely using a 'slow-sync' and 'two-way'
        clearLocalSyncData(sources)
        cmdPrefix="XDG_CACHE_HOME=%s XDG_CONFIG_HOME=%s " % (testResult, testConfig)
        if (not self.ctcap):
            cmdPrefix += "SYNCEVOLUTION_NOCTCAP=t "
        syncevoTest = "%s %s --daemon=no" % (cmdPrefix, syncevoCmd)
        syncCmd = "%s --sync slow %s %s" % (syncevoTest, configName, self.source)
        status,interrupt = runSync(syncCmd)
        if (not status or interrupt):
            return (status, interrupt)
        clearLocalSyncData(sources)
        syncCmd = "%s --sync two-way %s %s" % (syncevoTest, configName, self.source)
        status,interrupt = runSync(syncCmd)
        if (not status or interrupt):
            return (status, interrupt)

        #step 2: insert testcase to local data and sync with 'two-way'
        insertLocalSyncData(sources, self.type)
        syncCmd = "%s --sync two-way %s %s" % (syncevoTest, configName, self.source)
        status,interrupt = runSync(syncCmd)
        if (not status or interrupt):
            return (status, interrupt)

        #step 3: delete local data and sync with 'slow-sync'
        clearLocalSyncData(sources)
        syncCmd = "%s --sync slow %s %s" % (syncevoTest, configName, self.source)
        status,interrupt = runSync(syncCmd)
        if (not status or interrupt):
            return (status, interrupt)

        #step 4: compare the received data with test case
        status = compareSyncData(sources, self.type)
        return (status, interrupt)

    '''
    The test driver iterating all possible test combinations and try them one by one
    '''
    def run(self):
        #first round of iterating, calculating all possible configuration numbers
        allconfigs = 0
        for self.ctcap in self.ctcaps:
            for self.identifier in self.identifiers:
                for self.version in self.versions:
                    for self.source in self.sources:
                        for self.uri in self.uris[self.source]:
                            for self.type in self.types[self.source]:
                                allconfigs +=1
        print "Starting test for %d configurations..." %(allconfigs,)

        curconfig = 0
        self.wConfigs = {}
        for source in self.sources:
            self.wConfigs[source] = None

        #second round of iterating, test for each configuration
        interrupt = False
        for self.source in self.sources:
           if(interrupt):
               break
           for self.ctcap in self.ctcaps:
               if(interrupt):
                   break
               for self.version in self.versions:
                   if(interrupt):
                       break
                   for self.identifier in self.identifiers:
                       if(interrupt):
                           break
                       for self.uri in self.uris[self.source]:
                           if(interrupt):
                               break
                           for self.type in self.types[self.source]:
                               curconfig +=1
                               skip = self.prepare (allconfigs, curconfig)
                               if (not skip):
                                   (status, interrupt) = self.testWithCurrentConfiguration ()
                                   if (status and not interrupt):
                                       self.wConfigs[self.source] = ConfigurationParameter (self.version, self.source, self.uri, self.type, self.ctcap, self.identifier)
                                       print "Found a working configuration for %s" % (self.source,)
                                       if (options.verbose > 0):
                                           self.wConfigs[self.source].printMe()
                               if (interrupt):
                                   break;
        if(interrupt):
            print "Test Interrupted"
            return 1

        print "Test Ended"

        #Test finished, print summary and generating configurations
        print "****************SUMMARY****************"
        found = False
        for source,config in self.wConfigs.items():
            if (config):
                found = True
                print "------------------------------------------"
                print "Configuration parameter for %s:" % (source,)
                config.printMe()

        if (not found):
            print "No working configuration found"
        else:
            have_combined = \
                self.wConfigs.has_key('calendar') and \
                self.wConfigs.has_key('todo') and \
                self.wConfigs['calendar'] and \
                self.wConfigs['todo'] and \
                self.wConfigs['calendar'].uri == self.wConfigs['todo'].uri

            if (options.create):
                #first remove the previous configuration if there is a configuration with the same name
                create = options.create
                cmd = "%s --remove '%s'" %(syncevoCmd, create)
                try:
                    runCommand (cmd)
                except:
                    pass
                cmd = "%s -c --template 'SyncEvolution Client' --sync-property peerIsClient=1 %s" %(syncevoCmd, create)
                runCommand (cmd)
                #disable all sources by default
                for source in allSources:
                    if source == 'calendar+todo':
                        continue
                    cmd = "%s -c --source-property sync='disabled' %s %s" %(syncevoCmd, create, source)
                    runCommand(cmd)

                syncCreated = False
                for source,config in self.wConfigs.items():
                    if (config):
                        if (not syncCreated):
                            #set the sync parameter
                            cmd = "%s --configure --sync-property syncURL='obex-bt://%s' --sync-property remoteIdentifier='%s' --sync-property SyncMLVersion='%s' '%s'" %(syncevoCmd, self.btaddr, config.identifier, config.version, create)
                            syncCreated = True
                            runCommand (cmd)
                        #set each source parameter
                        ltype = strip_version(config.type)
                        cmd = "%s --configure --source-property sync='two-way' --source-property URI='%s' --source-property type='%s:%s' '%s' '%s'" %(syncevoCmd, config.uri, source, ltype, create, config.source)
                        runCommand(cmd)
                if have_combined:
                    ltype = strip_version(self.wConfigs['calendar'].type)
                    uri = self.wConfigs['calendar'].uri
                    cmd = "%s --configure --source-property evolutionsource='calendar,todo' --source-property sync='two-way' --source-property URI='%s' --source-property type='virtual:%s' '%s' calendar+todo" %(syncevoCmd, uri, ltype, create)
                    runCommand(cmd)
                    for source in ('calendar', 'todo'):
                        cmd = "%s --configure --source-property sync='none' --source-property URI='%s' '%s' %s" %(syncevoCmd, uri, create, source)
                        runCommand(cmd)


            if (options.advanced):
                print ""
                print "We have conducted basic test by sending and receiving"
                print "data to the phone. You can help the SyncEvolution project"
                print "and other users by submitting the following configuration"
                print "template at http://syncevolution.org/wiki/phone-compatibility-template"
                print ""

                configini = { "peerIsClient": "1" }
                sourceConfigInis = {}

                for source,config in self.wConfigs.items():
                    if(config):
                        sourceini = {}
                        if (config.identifier):
                            configini["remoteIdentifier"] = config.identifier
                        if (config.version != '1.2'):
                            configini["SyncMLVersion"] = config.version
                        sourceini["sync"] = "two-way"
                        sourceini["uri"] = config.uri
                        sourceini["backend"] = source
                        sourceini["syncFormat"] = strip_version(config.type)
                        sourceConfigInis[source] = sourceini

                # create 'calendar+todo' entry, disable separate 'calendar' and 'todo'?
                if have_combined:
                    sourceini = {}
                    sourceini["sync"] = "two-way"
                    sourceini["database"] = "calendar,todo"
                    sourceini["uri"] = self.wConfigs['calendar'].uri
                    sourceini["backend"] = "virtual"
                    sourceini["syncFormat"] = strip_version(self.wConfigs['calendar'].type)
                    sourceConfigInis['calendar+todo'] = sourceini
                    # disable the sub datasources
                    for source in ('calendar', 'todo'):
                        sourceConfigInis[source]["sync"] = "none"
                        sourceConfigInis[source].pop("uri")

                # print template to stdout
                sep = "--------------------> snip <--------------------"
                print sep
                print "=== template.ini ==="
                print "fingerprint = <Model> <Manufacturer>"
                print "=== config.ini ==="
                print hash2ini(configini)
                print "consumerReady = 1"
                for source, configini in sourceConfigInis.items():
                    print "=== sources/%s/config.ini ===" % source
                    print hash2ini(configini)
                print sep
            else:
                print ""
                print "We just conducted minimum test by syncing with the phone"
                print "without checking received data. For more reliable result,"
                print "use the --advanced option, but beware that it will overwrite"
                print "contacts, events, tasks and memos on the phone."

            if (options.create):
                print ""
                print "Created configuration: %s" %(options.create)
                print "You may start syncing with: syncevolution %s" %(options.create)

def main():
    versions = []
    sources = []
    ctcaps = []
    identifiers = []
    uris = {}
    types = {}
    if (options.version):
        versions.append (options.version)
    if (options.source):
        sources.append (options.source)
    if (options.uri):
        uris[sources[0]] = []
        uris[sources[0]].append(options.uri)
    if (options.type):
        types[sources[0]] = []
        types[sources[0]].append(options.type)
    if (options.ctcap):
        ctcaps.append (options.ctcap)
    if (options.identifier):
        identifiers.append (options.identifier)

    config = TestingConfiguration (versions, sources, uris, types, ctcaps,
            identifiers, options.btaddr)

    tmpdir = tempfile.mkdtemp(prefix="syncevo-phone-config")
    global testFolder
    global testResult
    global testConfig
    testFolder = tmpdir+'/data'
    testResult = tmpdir+'/cache'
    testConfig = tmpdir+'/config'
    print "Running test with test data inside %s and test results inside %s" %(testFolder, testResult)
    config.run()

if __name__ == "__main__":
  main()



#!/usr/bin/python

"""
Converts the .log output for a client-test test into HTML, with
hightlighting and local hrefs to ClientTest.html and test directories.
"""

import sys
import re
import cgi

filename = sys.argv[1]
if filename == '-':
    log = sys.stdin
else:
    log = open(filename)

out = sys.stdout

# matches [DEBUG/DEVELOPER/...] tags at the start of the line,
# used to mark text via <span class=mode>
class ModeSpan:
    mode=None

    @staticmethod
    def setMode(newmode):
        if ModeSpan.mode != newmode:
            if ModeSpan.mode:
                out.write('</span>')
            ModeSpan.mode = newmode
            if ModeSpan.mode:
                out.write('<span class="%s">' % ModeSpan.mode)

# detect SyncEvolution line prefix
tag = re.compile(r'^(\[([A-Z]+) [^\]]*\])( .*)')

# hyperlink to HTML version of source code
sourcefile = re.compile(r'((\w+\.(?:cpp|c|h)):(\d+))')

# highlight:
# - any text after ***
#   *** clean via source A
#   *** starting Client::Source::egroupware-dav_caldav::testChanges ***
# - simple prefixes at the start of a line, after the [] tag (removed already)
#    caldav #A:
# - HTTP requests
#   REPORT /egw/groupdav.php/syncevolution/calendar/ HTTP/1.1
#   PROPFIND /egw/groupdav.php/syncevolution/calendar/1234567890%40dummy.ics HTTP/1.1
highlight = re.compile(r'(\*\*\* .*|^ [a-zA-Z0-9_\- #]*: |(?:REPORT|PUT|GET|DELETE|PROPFIND) .*HTTP/\d\.\d$)')

# hyperlink to sync session directory
session = re.compile(r'(Client_(?:Source|Sync)(?:_\w+)+\S+)')

out.write('''<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
   <meta http-equiv="content-type" content="text/html; charset=None">
   <style type="text/css">
td.linenos { background-color: #f0f0f0; padding-right: 10px; }
span.lineno { background-color: #f0f0f0; padding: 0 5px 0 5px; }
pre { line-height: 125%; }
body .hll { background-color: #ffffcc }
body  { background: #f8f8f8; }
span.INFO { background: #c0c0c0 }
span.ERROR { background: #e0c0c0 }
span.hl { color: #c02020 }
   </style>
</head>
<body>
<pre>''')

def simplifyFilename(test):
    "same as client-test-main.cpp simplifyFilename()"
    test = test.replace(':', '_')
    test = test.replace('__', '_')
    return test

for line in log:
    line = line.rstrip()
    m = tag.match(line)
    if m:
        ModeSpan.setMode(m.group(2))
        out.write('<span class="tag">%s</span>' % m.group(1))
        line = m.group(3)
    line = cgi.escape(line)
    line = sourcefile.sub(r'<a href="\2.html#True-\3">\1</a>', line)
    line = highlight.sub(r'<span class="hl">\1</span>', line)
    line = session.sub(r'<a href="\1/">\1</a>', line)
    out.write(line)
    out.write('\n')

# close any <span class=mode> opened before
ModeSpan.setMode(None)

out.write('''
</pre>
</body>
</html>''')



#!/usr/bin/python

"""
Converts source code (first parameter, can be - for stdin) to HTML
(stdout), using pygments if installed, otherwise simple text
manipulation without syntax highlighting. In both cases the output
will have "True-<number>" as anchors for each line.
"""

import sys

filename = sys.argv[1]
if filename == '-':
    code = sys.stdin.read()
else:
    code = open(filename).read()

out = sys.stdout

try:
    import pygments
    import pygments.lexers
    from pygments.formatters import HtmlFormatter
    if filename == '-':
        lexer = pygments.lexers.guess_lexer(code)
    else:
        lexer = pygments.lexers.guess_lexer_for_filename(filename, code)
    formatter = HtmlFormatter(full=True, linenos=True, lineanchors=True)
    pygments.highlight(code, lexer, formatter, out)
except:
    import cgi
    print >>sys.stderr, "source2html.py failed with pygments:", sys.exc_info()
    print >>sys.stderr, "falling back to internal code"

    out.write('''<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
   <meta http-equiv="content-type" content="text/html; charset=None">
   <style type="text/css">
td.linenos { background-color: #f0f0f0; padding-right: 10px; }
span.lineno { background-color: #f0f0f0; padding: 0 5px 0 5px; }
pre { line-height: 125%; }
body .hll { background-color: #ffffcc }
body .hll { background-color: #ffffcc }
body  { background: #f8f8f8; }
   </style>
</head>
<body>
<table class="highlighttable"><tr><td class="linenos"><div class="linenodiv"><pre>''')
    lines = code.split('\n')
    for line in range(1, len(lines)):
        out.write('%4d\n' % line)
    out.write('%4d' % len(lines))
    out.write('</pre></div></td><td class="code"><div class="highlight"><pre>')
    for lineno, line in enumerate(lines):
        out.write('<a name="True-%d"></a>%s\n' % (lineno + 1, cgi.escape(line)))
    out.write('''</pre></div>
</td></tr></table></body>
</html>''')



#!/usr/bin/python

'''
 Copyright (C) 2009 Intel Corporation

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) version 3.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301  USA
'''
import sys,os,glob,datetime
import re
import fnmatch
import cgi
import subprocess

""" 
resultcheck.py: tranverse the test result directory, generate an XML
based test report.
"""

# sort more accurately on sub-second modification times
os.stat_float_times(True)

space="  "
def check (resultdir, serverlist,resulturi, srcdir, shellprefix, backenddir):
    '''Entrypoint, resutldir is the test result directory to be generated,
    resulturi is the http uri, it will only process corresponding server's
    test results list in severlist'''
    if serverlist:
        servers = serverlist.split(",")
    else:
        servers = []
    result = open("%s/nightly.xml" % resultdir,"w")
    result.write('''<?xml version="1.0" encoding="utf-8" ?>\n''')
    result.write('''<nightly-test>\n''')
    indents=[space]
    if(os.path.isfile(resultdir+"/output.txt")==False):
        print "main test output file not exist!"
    else:
        indents,cont = step1(resultdir,result,indents,resultdir,resulturi, shellprefix, srcdir)
        if (cont):
            step2(resultdir,result,servers,indents,srcdir,shellprefix,backenddir)
        else:
            # compare.xsl fails if there is no <client-test> element:
            # add an empty one
            result.write('''<client-test/>\n''')
    result.write('''</nightly-test>\n''')
    result.close()

patchsummary = re.compile('^Subject: (?:\[PATCH.*?\] )?(.*)\n')
patchauthor = re.compile('^From: (.*?) <.*>\n')
def extractPatchSummary(patchfile):
    author = ""
    for line in open(patchfile):
        m = patchauthor.match(line)
        if m:
            author = m.group(1) + " - "
        else:
            m = patchsummary.match(line)
            if m:
                return author + m.group(1)
    return os.path.basename(patchfile)

def step1(resultdir, result, indents, dir, resulturi, shellprefix, srcdir):
    '''Step1 of the result checking, collect system information and 
    check the preparation steps (fetch, compile)'''

    # Always keep checking, even if any of the preparation steps failed.
    cont = True

    input = os.path.join(resultdir, "output.txt")
    indent =indents[-1]+space
    indents.append(indent)

    # include information prepared by GitCopy in runtests.py
    result.write(indent+'<source-info>\n')
    files = os.listdir(resultdir)
    files.sort()
    for source in files:
        m = re.match('(.*)-source.log', source)
        if m:
            name = m.group(1)
            result.write('   <source name="%s"><description><![CDATA[%s]]></description>\n' %
                         (name, open(os.path.join(resultdir, source)).read()))
            result.write('       <patches>\n')
            for patch in files:
                if fnmatch.fnmatch(patch, name + '-*.patch'):
                    result.write('          <patch><path>%s</path><summary><![CDATA[%s]]></summary></patch>\n' %
                                 ( patch, extractPatchSummary(os.path.join(resultdir, patch)) ) )
            result.write('       </patches>\n')
            result.write('   </source>\n')
    result.write(indent+'</source-info>\n')

    result.write(indent+'''<platform-info>\n''')
    indent =indents[-1]+space
    indents.append(indent)
    result.write(indent+'''<cpuinfo>\n''')
    fout=subprocess.check_output('cat /proc/cpuinfo|grep "model name" |uniq', shell=True)
    result.write(indent+fout)
    result.write(indent+'''</cpuinfo>\n''')
    result.write(indent+'''<memoryinfo>\n''')
    fout=subprocess.check_output('cat /proc/meminfo|grep "Mem"', shell=True)
    for s in fout.split('\n'):
        result.write(indent+s)
    result.write(indent+'''</memoryinfo>\n''')
    result.write(indent+'''<osinfo>\n''')
    fout=subprocess.check_output('uname -osr'.split())
    result.write(indent+fout)
    result.write(indent+'''</osinfo>\n''')
    if 'schroot' in shellprefix:
        result.write(indent+'''<chrootinfo>\n''')
        # Don't make assumption about the schroot invocation. Instead
        # extract the schroot name from the environment of the shell.
        name=subprocess.check_output(shellprefix + "sh -c 'echo $SCHROOT_CHROOT_NAME'",
                                     shell=True)
        info = re.sub(r'schroot .*', 'schroot -i -c ' + name, shellprefix)
        fout=subprocess.check_output(info, shell=True)
        s = []
        for line in fout.split('\n'):
            m = re.match(r'^\s+(Name|Description)\s+(.*)', line)
            if m:
                s.append(indent + m.group(1) + ': ' + m.group(2))
        result.write('\n'.join(s))
        result.write(indent+'''</chrootinfo>\n''')
    result.write(indent+'''<libraryinfo>\n''')
    libs = ['libsoup-2.4', 'evolution-data-server-1.2', 'glib-2.0','dbus-glib-1']
    s=''
    for lib in libs:
        try:
            fout=subprocess.check_output(shellprefix+' pkg-config --modversion '+lib +' |grep -v pkg-config',
                                         shell=True)
            s = s + lib +': '+fout +'  '
        except subprocess.CalledProcessError:
            pass
    result.write(indent+s)
    result.write(indent+'''</libraryinfo>\n''')
    indents.pop()
    indent = indents[-1]
    result.write(indent+'''</platform-info>\n''')
    result.write(indent+'''<prepare>\n''')
    indent =indent+space
    indents.append(indent)
    tags=['libsynthesis', 'syncevolution', 'activesyncd', 'compile', 'dist', 'distcheck']
    tagsp={'libsynthesis':'libsynthesis-source',
           'syncevolution':'syncevolution-source',
           'activesyncd':'activesyncd-source',
           'compile':'compile',
           'distcheck': 'distcheck',
           'dist':'dist'}
    for tag in tags:
        result.write(indent+'''<'''+tagsp[tag])
        fout=subprocess.check_output('find `dirname '+input+'` -type d -name *'+tag, shell=True)
        s = fout.rpartition('/')[2].rpartition('\n')[0]
        result.write(' path ="'+s+'">')
	'''check the result'''
        if 0 == os.system("grep -q '^"+tag+": .*: failed' "+input):
            result.write("failed")
        elif 0 == os.system ("grep -q '^"+tag+" successful' "+input):
            result.write("okay")
        elif 0 == os.system("grep -q '^"+tag+".* disabled in configuration$' "+input):
            result.write("skipped")
        else:
            # Not listed at all? Fail.
            result.write("failed")
        result.write('''</'''+tagsp[tag]+'''>\n''')
    indents.pop()
    indent = indents[-1]
    result.write(indent+'''</prepare>\n''')
    result.write(indent+'''<log-info>\n''')
    indent =indent+space
    indents.append(indent)
    result.write(indent+'''<uri>'''+resulturi+'''</uri>\n''')
    indents.pop()
    indent = indents[-1]
    result.write(indent+'''</log-info>\n''')
    indents.pop()
    indent = indents[-1]
    return (indents, cont)

def step2(resultdir, result, servers, indents, srcdir, shellprefix, backenddir):
    '''Step2 of the result checking, for each server listed in
    servers, tranverse the corresponding result folder, process
    each log file to decide the status of the testcase'''
    '''Read the runtime parameter for each server '''
    params = {}
    if servers:
        cmd='sed -n '
        for server in servers:
            cmd+= '-e /^'+server+'/p '
        print "Analyzing overall result %s" % (resultdir+'/output.txt')
        cmd = cmd +resultdir+'/output.txt'
        fout=subprocess.check_output(cmd, shell=True)
        for line in fout.split('\n'):
            for server in servers:
                # Find first line with "foobar successful" or "foobar: <command failure>",
                # ignore "skipped".
                if (line.startswith(server + ":") or line.startswith(server + " ")) and server not in params:
                    t = line.partition(server)[2]
                    if(t.startswith(':')):
                        t=t.partition(':')[2]
                    t = t.strip()
                    if t != 'skipped: disabled in configuration':
                        print "Result for %s: %s" % (server, t)
                        params[server]=t

    indent =indents[-1]+space
    indents.append(indent)
    '''start of testcase results '''
    result.write(indent+'''<client-test>\n''')
    runservers = os.listdir(resultdir)
    #list source test servers statically, we have no idea how to differenciate
    #automatically whether the server is a source test or sync test.
    sourceServers = ['evolution',
                     'eds',
                     'kde',
                     'file',
                     'unittests',
                     'evolution-prebuilt-build',
                     'yahoo',
                     'owndrive',
                     'davical',
                     'googlecalendar',
                     'googlecontacts',
                     'googleeas',
                     'apple',
                     'egroupware-dav',
                     'oracle',
                     'exchange',
                     'pim',
                     'dbus']
    sourceServersRun = 0
    haveSource = False
    #Only process servers listed in the input parameter and in the sourceServer
    #list and have a result folder
    for server in servers:
        matched = False
        for rserver in runservers:
            for source in sourceServers:
                if (rserver.find('-')!=-1 and server == rserver.partition('-')[2] and server == source):
                    matched = True
                    break
        if(matched):
            #put the servers at the front of the servers list, so that we will
            #process test first
            servers.remove(server)
            servers.insert (0, server);
            sourceServersRun = sourceServersRun+1;
            haveSource = True

    #process source tests first 
    if (haveSource) :
        indent +=space
        indents.append(indent)
        result.write(indent+'''<source>\n''')

    haveSync = False
    for server in servers:
        matched = False
	'''Only process servers listed in the input parametr'''
        for rserver in runservers:
            if(rserver.find('-')!= -1 and rserver.partition('-')[2] == server):
                matched = True
                break;
        if(matched):
            sourceServersRun = sourceServersRun -1;
            if (sourceServersRun == -1):
                haveSync = True
                '''generate a template which lists all test cases we supply, this helps 
                generate a comparable table and track potential uncontentional skipping
                of test cases'''
                templates=[]
                oldpath = os.getcwd()
                # Get list of Client::Sync tests one source at a time (because
                # the result might depend on CLIENT_TEST_SOURCES and which source
                # is listed there first) and combine the result for the common
                # data types (because some tests are only enable for contacts, others
                # only for events).
                # The order of the tests matters, so don't use a hash and start with
                # a source which has only the common tests enabled. Additional tests
                # then get added at the end.
                clientSync = re.compile(r' +Client::Sync::(.*?)::(?:(Suspend|Resend|Retry)::)?([^:]+)')
                for source in ('file_task', 'file_event', 'file_contact', 'eds_contact', 'eds_event'):
                    cmd = shellprefix + " env LD_LIBRARY_PATH=%s/build-synthesis/src/.libs SYNCEVOLUTION_BACKEND_DIR=%s CLIENT_TEST_PEER_CAN_RESTART=1 CLIENT_TEST_RETRY=t CLIENT_TEST_RESEND=t CLIENT_TEST_SUSPEND=t CLIENT_TEST_SOURCES=%s %s/client-test -h" % (srcdir, backenddir, source, srcdir)
                    fout=subprocess.check_output(cmd, shell=True)
                    for line in fout.split('\n'):
                        m = clientSync.match(line)
                        if m:
                            if m.group(2):
                                # special sub-grouping
                                test = m.group(2) + '__' + m.group(3)
                            else:
                                test = m.group(3)
                            if test and test not in templates:
                                templates.append(test)
                indent +=space
                indents.append(indent)
                result.write(indent+'<sync>\n')
                result.write(indent+space+'<template>')
                for template in templates:
                    result.write(indent+space+'<'+template+'/>')
                result.write('</template>\n')
            indent +=space
            indents.append(indent)
            result.write(indent+'<'+server+' path="' +rserver+'" ')
            #valgrind check resutls
            if not params.get(server, None):
                # Unknown result, treat as failure.
                result.write('result="1"')
            else:
                m = re.search(r'return code (\d+)', params[server])
                if m:
                    result.write('result="%s"' % m.group(1))
            result.write('>\n')
            # sort files by creation time, to preserve run order
            logs = map(lambda file: (os.stat(file).st_mtime, file),
                       glob.glob(resultdir+'/'+rserver+'/*.log'))
            logs.sort()
            logs = map(lambda entry: entry[1], logs)
            logdic ={}
            logprefix ={}
            if server in ('dbus', 'pim'):
                # Extract tests and their results from output.txt,
                # which contains Python unit test output. Example follows.
                # Note that there can be arbitrary text between the test name
                # and "ok" resp. "FAIL/ERROR". Therefore failed tests
                # are identified not by those words but rather by the separate
                # error reports at the end of the output. Those reports
                # are split out into separate .log files for easy viewing
                # via the .html report.
                #
                # TestDBusServer.testCapabilities - Server.Capabilities() ... ok
                # TestDBusServer.testGetConfigScheduleWorld - Server.GetConfigScheduleWorld() ... ok
                # TestDBusServer.testGetConfigsEmpty - Server.GetConfigsEmpty() ... ok
                # TestDBusServer.testGetConfigsTemplates - Server.GetConfigsTemplates() ... FAIL
                # TestDBusServer.testInvalidConfig - Server.NoSuchConfig exception ... ok
                # TestDBusServer.testVersions - Server.GetVersions() ... ok
                #
                #======================================================================
                # FAIL: TestDBusServer.testGetConfigsTemplates - Server.GetConfigsTemplates()
                # ---------------------------------------------------------------------
                #
                # More recent Python 2.7 produces:
                # FAIL: testSyncSecondSession (__main__.TestSessionAPIsReal)

                # first build list of all tests, assuming that they pass
                dbustests = {}
                test_start = re.compile(r'''^Test(?P<cl>.*)\.test(?P<func>[^ ]*).*ok(?:ay)?$''')
                # FAIL/ERROR + description of test (old Python)
                test_fail = re.compile(r'''(?P<type>FAIL|ERROR): Test(?P<cl>.*)\.test(?P<func>[^ ]*)''')
                # FAIL/ERROR + function name of test (Python 2.7)
                test_fail_27 = re.compile(r'''(?P<type>FAIL|ERROR): test(?P<func>[^ ]*) \(.*\.(?:Test(?P<cl>.*))\)''')
                name = None
                logfile = None
                htmlfile = None
                linetype = None
                for line in open(rserver + "/output.txt"):
                    m = test_start.search(line)
                    if m:
                        is_okay = True
                    else:
                        m = test_fail.search(line) or test_fail_27.search(line)
                        is_okay = False
                    if m:
                        # create new (?!) log file
                        cl = m.group("cl")
                        func = m.group("func")
                        newname = rserver + "/" + cl + "_" + func + ".log"
                        if newname != name:
                            name = newname
                            logfile = open(name, "w")
                            if htmlfile:
                                htmlfile.write('</pre></body')
                                htmlfile.close()
                                htmlfile = None
                            htmlfile = open(name + ".html", "w")
                            htmlfile.write('''<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
   <meta http-equiv="content-type" content="text/html; charset=None">
   <style type="text/css">
td.linenos { background-color: #f0f0f0; padding-right: 10px; }
span.lineno { background-color: #f0f0f0; padding: 0 5px 0 5px; }
pre { line-height: 125%; }
body .hll { background-color: #ffffcc }
body  { background: #f8f8f8; }
span.INFO { background: #c0c0c0 }
span.ERROR { background: #e0c0c0 }
span.hl { color: #c02020 }
   </style>
</head>
<body>
<pre>''')
                            if not dbustests.get(cl):
                                dbustests[cl] = {}
                            if is_okay:
                                # okay: write a single line with the full test description
                                dbustests[cl][func] = "okay"
                                logfile.write(line)
                                logfile = None
                                htmlfile.write('<span class="OKAY">%s</span></pre></body>' % cgi.escape(line))
                                htmlfile.close()
                                htmlfile = None
                            else:
                                # failed: start writing lines into separate log file
                                dbustests[cl][func] = m.group("type")
                                linetype = "ERROR"
                                htmlfile.write('<a href="#dbus-traffic">D-Bus traffic</a> <a href="#stdout">output</a>\n\n')

                    if logfile:
                        logfile.write(line)
                        if line == 'D-Bus traffic:\n':
                            linetype = "DBUS"
                            htmlfile.write('<h3 id="dbus-traffic">D-Bus traffic:</h3>')
                        elif line == 'server output:\n':
                            linetype = "OUT"
                            htmlfile.write('<h3 id="stdout">server output:</h3>')
                        else:
                            htmlfile.write('<span class="%s">%s</span>' % (linetype, cgi.escape(line)))

                if htmlfile:
                    htmlfile.write('</pre></body')
                    htmlfile.close()
                    htmlfile = None

                # now write XML
                indent +=space
                indents.append(indent)
                for testclass in dbustests:
                    result.write('%s<%s prefix="">\n' %
                                 (indent, testclass))
                    indent +=space
                    indents.append(indent)
                    for testfunc in dbustests[testclass]:
                        result.write('%s<%s>%s</%s>\n' %
                                     (indent, testfunc,
                                      dbustests[testclass][testfunc], 
                                      testfunc))
                    indents.pop()
                    indent = indents[-1]
                    result.write('%s</%s>\n' %
                                 (indent, testclass))
                indents.pop()
                indent = indents[-1]
            else:
                for log in logs:
                    logname = os.path.basename(log)
                    if logname in ['____compare.log',
                                   'syncevo.log', # D-Bus server output
                                   'dbus.log', # dbus-monitor output
                                   ]:
                        continue
                    # <path>/Client_Sync_eds_contact_testItems.log
                    # <path>/SyncEvo_CmdlineTest_testConfigure.log
                    # <path>/N7SyncEvo11CmdlineTestE_testConfigure.log - C++ name mangling?
                    m = re.match(r'^(Client_Source_|Client_Sync_|N7SyncEvo\d+|[^_]*_)(.*)_([^_]*)\.log', logname)
                    if not m or logname.endswith('.server.log'):
                        print "skipping", logname
                        continue
                    # Client_Sync_, Client_Source_, SyncEvo_, ...
                    prefix = m.group(1)
                    # eds_contact, CmdlineTest, ...
                    format = m.group(2)
                    # testImport
                    casename = m.group(3)
                    # special case grouping of some tests: include group inside casename instead of
                    # format, example:
                    # <path>/Client_Source_apple_caldav_LinkedItemsDefault_testLinkedItemsParent
                    m = re.match(r'(.*)_(LinkedItems\w+|Suspend|Resend|Retry)', format)
                    if m:
                        format = m.group(1)
                        casename = m.group(2) + '::' + casename
                        if '.' in casename:
                            # Ignore sub logs.
                            print "skipping", logname
                            continue
                    # Another special case: suspend/resend/retry uses an intermediate grouping
                    # which we can ignore because the name is repeated in the test case name.
                    # m = re.match(r'(.*)_(Suspend|Resend|Retry)', format)
                    # if m:
                    #     format = m.group(1)
                    #     # Case name *not* extended, in contrast to the
                    #     # LinkedItems case above.
                    #     if '.' in casename:
                    #         # Ignore sub logs.
                    #         print "skipping", logname
                    #         continue
                    print "analyzing log %s: prefix %s, subset %s, testcase %s" % (logname, prefix, format, casename)
                    if(format not in logdic):
                        logdic[format]=[]
                    logdic[format].append((casename, log))
                    logprefix[format]=prefix
            for format in logdic.keys():
                indent +=space
                indents.append(indent)
                prefix = logprefix[format]
                qformat = format;
                # avoid + sign in element name (not allowed by XML);
                # code reading XML must replace _- with + and __ with _
                qformat = qformat.replace("_", "__");
                qformat = qformat.replace("+", "_-");
                result.write(indent+'<'+qformat+' prefix="'+prefix+'">\n')
                for casename, log in logdic[format]:
                    indent +=space
                    indents.append(indent)
                    # must avoid :: in XML
                    tag = casename.replace('::', '__')
                    result.write(indent+'<'+tag+'>')
                    match=format+'::'+casename
                    matchOk=match+": okay \*\*\*"
                    matchKnownFailure=match+": \*\*\* failure ignored \*\*\*"
                    if not os.system("grep -q '" + matchKnownFailure + "' "+log):
                       result.write('knownfailure')
                    elif not os.system("tail -10 %s | grep -q 'external transport failure (local, status 20043)'" % log):
                        result.write('network')
                    elif os.system("grep -q '" + matchOk + "' "+log):
                       result.write('failed')
                    else:
                        result.write('okay')
                    result.write('</'+tag+'>\n')
                    indents.pop()
                    indent = indents[-1]
                result.write(indent+'</'+qformat+'>\n')
                indents.pop()
                indent = indents[-1]
            result.write(indent+'</'+server+'>\n')
            indents.pop()
            indent = indents[-1]
            if(sourceServersRun == 0):
                #all source servers have been processed, end the source tag and
                #start the sync tags
                result.write(indent+'''</source>\n''')
                indents.pop()
                indent = indents[-1]
    if(haveSync):
        result.write(indent+'</sync>\n')
        indents.pop()
        indent=indents[-1]
    result.write(indent+'''</client-test>\n''')
    indents.pop()
    indents = indents[-1]

if(__name__ == "__main__"):
    if (len(sys.argv)!=7):
        # srcdir and basedir must be usable inside the shell started by shellprefix (typically
        # the chroot).
        print "usage: python resultchecker.py resultdir servers resulturi srcdir shellprefix backenddir"
    else:
        check(*sys.argv[1:])



#! /usr/bin/python

'''Runs a sync.
Usage: dbus-server-sync <server> <mode> <source modes>
<server> - configuration name
<mode> - "", "two-way", "disabled", ...
<source modes> - "{}" or Python hash (like {"addressbook": "refresh-from-server"})
'''

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject
import sys

DBusGMainLoop(set_as_default=True)

bus = dbus.SessionBus()

object = dbus.Interface(bus.get_object('org.syncevolution',
                                       '/org/syncevolution/Server'),
                        'org.syncevolution.Server')

loop = gobject.MainLoop()

sessionpath = None
def SessionChanged(object, ready):
    print "SessionChanged:", object, ready
    if sessionpath == object:
        loop.quit()

bus.add_signal_receiver(SessionChanged,
                        'SessionChanged',
                        'org.syncevolution.Server',
                        'org.syncevolution',
                        None,
                        byte_arrays=True)

dummysessionpath = object.StartSession("")
sessionpath = object.StartSession(sys.argv[1])

# detach from dummy session so that real session can run
session = dbus.Interface(bus.get_object('org.syncevolution',
                                        dummysessionpath),
                         'org.syncevolution.Session')
session.Detach()
session = dbus.Interface(bus.get_object('org.syncevolution',
                                        sessionpath),
                         'org.syncevolution.Session')

print 'session created:', session.GetStatus(), session.GetProgress()
# wait for session ready
loop.run()
print 'session ready:', session.GetStatus(), session.GetProgress()
session.Sync(sys.argv[2], eval(sys.argv[3]))
print 'sync started:', session.GetStatus(), session.GetProgress()
# wait for session done
loop.run()
print 'done:', session.GetStatus(), session.GetProgress()



#! /usr/bin/python -u

# * Copyright (C) 2011 Intel Corporation

# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 or 3.0 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import dbus
import string
import sys

# Defines
SYNCML_UUID  = "00000002-0000-1000-8000-0002ee000002"
PNPINFO_UUID = "00001200-0000-1000-8000-00805f9b34fb"
PNPINFO_ATTRIB = "0x1200"
SOURCE_ATTRIB  = "0x0205"
VENDOR_ATTRIB  = "0x0201"
PRODUCT_ATTRIB = "0x0202"

def extractValueFromServiceRecord(servRec, attribId):

    pos = string.find(servRec, attribId)

    if(pos < 0):
        return ""

    pos = string.find(servRec, "value", pos + len(attribId))
    pos = string.find(servRec, '"', pos) + 1
    endPos = string.find(servRec,'"', pos)
    return servRec[pos:endPos]

def getVendorAndProductId(pnpInfoServRec):
    '''Get the vendor and product ids from xml formatted service record.'''

    servRec = pnpInfoServRec.values()[0]

    sourceVal  = extractValueFromServiceRecord(servRec, SOURCE_ATTRIB)
    vendorVal  = extractValueFromServiceRecord(servRec, VENDOR_ATTRIB)
    productVal = extractValueFromServiceRecord(servRec, PRODUCT_ATTRIB)
    return (sourceVal, vendorVal, productVal)

def writeDeviceInfoToFile(ids, vendor, product, hasSyncML):

    filename = "syncevo-phone-info-[%s].txt" % product
    FILE = open(filename,"w")
    FILE.write("Thanks, for helping us improve phone syncing on Linux.\n")
    FILE.write("Please send this file or its contents to blixtra [at] gmail.com\n\n" )
    FILE.write("SyncML support: %s\n" % hasSyncML)
    if(len(ids) > 0):
        FILE.write("Source: %s\n"     % (ids[0]))
        FILE.write("Vendor: %s=%s\n"  % (ids[1], vendor))
        FILE.write("product: %s=%s\n\n" % (ids[2], product))
    else:
        FILE.write("Vendor: %s\n"  % vendor)
        FILE.write("product: %s\n\n" % product)
        FILE.write("This phone doesn't support the bluetooth Device ID profile.\n" )

    FILE.close()
    return filename

# Start main program
bus = dbus.SystemBus()
bluezIface = dbus.Interface(bus.get_object('org.bluez', '/'),
                            'org.bluez.Manager')

hasSyncmlSupport  = False
hasPnpInfoSupport = False
ids = {}

adapters = bluezIface.ListAdapters()
for adapter in adapters:
    adapterIface = dbus.Interface(bus.get_object('org.bluez', adapter),
                                  'org.bluez.Adapter')
    devices = adapterIface.ListDevices()
    for device in devices:
        try:
            deviceIface = dbus.Interface(bus.get_object('org.bluez', device),
                                         'org.bluez.Device')
            props = deviceIface.GetProperties();
            uuids = props["UUIDs"]
            print "Device name:", props.get("Name", "???")
            print "MAC Address:", props.get("Address", "???")
            for uuid in uuids:
                if SYNCML_UUID == uuid:
                    hasSyncmlSupport = True
                    print "   Supports SyncML."
                if PNPINFO_UUID == uuid:
                    hasPnpInfoSupport = True
                    print "   Looking up device information..."
                    sys.stdout.flush()
                    serviceRecord = deviceIface.DiscoverServices(PNPINFO_ATTRIB)
                    ids = getVendorAndProductId(serviceRecord)

            vendor  = raw_input("   What company makes this phone? (examples: Nokia, Sony Ericsson), empty to skip: ")
            if vendor:
                product = raw_input("   What is the model of this phone? (example: N900, K750i), empty to skip: ")
                if product:
                    # Write the results to a file
                    filename = writeDeviceInfoToFile(ids, vendor, product, hasSyncmlSupport)
                    print "Thanks, please send the file %s to blixtra [at] gmail.com" % filename
        except dbus.exceptions.DBusException, ex:
            print "   Failed, skipping device: %s" % ex



import dbus.service
import os

class Localed(dbus.service.Object):
    """a fake localed systemd implementation"""

    LOCALED_INTERFACE = "org.freedesktop.locale1"
    LOCALED_NAME = "org.freedesktop.locale1"
    LOCALED_PATH = "/org/freedesktop/locale1"
    LOCALED_LOCALE = "Locale"

    def __init__(self, bus=None):
        if bus is None:
            bus = dbus.SystemBus()
        bus_name = dbus.service.BusName(self.LOCALED_NAME, bus=bus)
        dbus.service.Object.__init__(self, bus_name, self.LOCALED_PATH)

        locale = []
        for name in (
            "LANG",
            "LC_CTYPE",
            "LC_NUMERIC",
            "LC_TIME",
            "LC_COLLATE",
            "LC_MONETARY",
            "LC_MESSAGES",
            "LC_PAPER",
            "LC_NAME",
            "LC_ADDRESS",
            "LC_TELEPHONE",
            "LC_MEASUREMENT",
            "LC_IDENTIFICATION",
            ):
            value = os.environ.get(name, None)
            if value != None:
                locale.append('%s=%s' % (name, value))

        self.properties = {
            self.LOCALED_LOCALE : locale,
            }

    def SetLocale(self, locale, invalidate=False):
        self.properties[self.LOCALED_LOCALE] = locale
        if invalidate:
            self.PropertiesChanged(self.LOCALED_INTERFACE,
                                   { },
                                   [ self.LOCALED_LOCALE ])
        else:
            self.PropertiesChanged(self.LOCALED_INTERFACE,
                                   { self.LOCALED_LOCALE: locale },
                                   [])

    # Properties
    @dbus.service.method(dbus.PROPERTIES_IFACE,
                         in_signature='ss',
                         out_signature='v')
    def Get(self, interface_name, property_name):
        return self.GetAll(interface_name)[property_name]

    @dbus.service.method(dbus.PROPERTIES_IFACE,
                         in_signature='s',
                         out_signature='a{sv}')
    def GetAll(self, interface_name):
        if interface_name == self.LOCALED_INTERFACE:
            return self.properties
        else:
            raise dbus.exceptions.DBusException(
                'org.syncevolution.UnknownInterface',
                'The fake localed object does not implement the %s interface'
                % interface_name)

    @dbus.service.signal(dbus.PROPERTIES_IFACE,
                         signature='sa{sv}as')
    def PropertiesChanged(self, interface_name, changed_properties,
                          invalidated_properties):
        pass



#! /usr/bin/python

'''Get config/reports and set config.
Usage: dbus-server-sync <server> [--getconfig | --config | --reports] [config options] 
<server> - configuration name
'''

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject
import sys

DBusGMainLoop(set_as_default=True)

bus = dbus.SessionBus()

object = dbus.Interface(bus.get_object('org.syncevolution',
                                       '/org/syncevolution/Server'),
                        'org.syncevolution.Server')

loop = gobject.MainLoop()

sessionpath = None
def SessionChanged(object, ready):
    print "SessionChanged:", object, ready
    if sessionpath == object:
        loop.quit()

bus.add_signal_receiver(SessionChanged,
                        'SessionChanged',
                        'org.syncevolution.Server',
                        'org.syncevolution',
                        None,
                        byte_arrays=True)

dummysessionpath = object.StartSession("")
sessionpath = object.StartSession(sys.argv[1])

# detach from dummy session so that real session can run
session = dbus.Interface(bus.get_object('org.syncevolution',
                                        dummysessionpath),
                         'org.syncevolution.Session')
session.Detach()
session = dbus.Interface(bus.get_object('org.syncevolution',
                                        sessionpath),
                         'org.syncevolution.Session')
if sys.argv[2] == "--getconfig":
    print 'Get Config:'
    dict = session.GetConfig(0)
    for name, value in dict.items():
        print 'name: [', name, ']'
        for config, confvalue in value.items():
            print '\t', config, ' = ', confvalue 
elif sys.argv[2] == "--config":
    print 'Set Config:'
    i = 3;
    length = len(sys.argv)
    dict = {}
    update = 1
    temporary = 0
    while i < length:
        if sys.argv[i] == "--clear":
            update = 0
            i = i + 1
            continue
        elif sys.argv[i] == "--temp":
            temporary = 1
            i = i + 1
            continue
        elif sys.argv[i].startswith('-z'):
            l1 = sys.argv[i].split('=')
            if len(l1) == 2:
                name = 'source/' + l1[1]
        elif sys.argv[i].startswith('-y'):
            name = ''

        l2 = sys.argv[i+1].split('=')
        idict = {}
        if dict.has_key(name):
            idict = dict[name]
            idict[l2[0]] = l2[1]
        else:
            idict[l2[0]] = l2[1]
            dict[name] = idict
        i = i+2

    print dict
    session.SetConfig(update, temporary, dict)
    session.Sync("", {})
elif sys.argv[2] == "--reports":
    reports = session.GetReports(0, 10)
    i = len(reports)
    j = 0
    while j < i:
        print 'Report ', j, ':'
        dict = reports.pop()
        for key, value in dict.items():
            print '\t[',key,'] = ' ,value
        j = j+1
    print ''
elif sys.argv[2] == "--checksource":
    i = 3;
    length = len(sys.argv)
    print 'CheckSource:'
    while i < length:
        try:
            session.CheckSource(sys.argv[i])
        except dbus.exceptions.DBusException, x:
            print '\t[', sys.argv[i], ']: failed, ', x
            i = i + 1
            continue
        print '\t[', sys.argv[i], ']: ok'
        i = i + 1
elif sys.argv[2] == "--getdatabases":
    i = 3;
    length = len(sys.argv)
    print 'GetDatabases:'
    while i < length:
        r = session.GetDatabases(sys.argv[i])
        print '\t[', sys.argv[i],']'
        for item in r:
            print '\t\t','name =', item[0], ', uri = ', item[1], ', default =', item[2]
        i = i + 1
    print ''
elif sys.argv[2] == "--getconfigs":
    i = 3
    length = len(sys.argv)
    r = {}
    if length == 3:
        r = session.GetConfigs(1)
        print ' Available configuration templates:'
    else:
        r = session.GetConfigs(0)
        print ' Configured servers:'
    for item in r:
        print '\t', item



#! /usr/bin/python

'''Usage: dbus-server-http.py <server name>
Runs a sync session with an HTTP SyncML server
configured in <server name> config.'''

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject
import sys
import httplib, urlparse

DBusGMainLoop(set_as_default=True)

bus = dbus.SessionBus()

object = dbus.Interface(bus.get_object('org.syncevolution',
                                       '/org/syncevolution/Server'),
                        'org.syncevolution.Server')
conpath = object.Connect({'description': 'dbus-server-connection.py',
                          'transport': 'HTTP'},
                         False,
                         0)
connection = dbus.Interface(bus.get_object('org.syncevolution',
                                           conpath),
                            'org.syncevolution.Connection')

loop = gobject.MainLoop()

def Abort():
    print "connection went down"

def Reply(data, type, meta, final, session):
    try:
        if final:
            print "closing connection"
            connection.Close(True, "")
        else:
            print ("send %d bytes of type %s, " % (len(data), type)), meta
            url = urlparse.urlparse(meta["URL"])
        
            if url.scheme == "http":
                conn = httplib.HTTPConnection(url.netloc)
            elif url.scheme == "https":
                conn = httplib.HTTPSConnection(url.netloc)
            else:
                raise "invalid scheme " + url.scheme
            conn.request("POST",
                         url.path,
                         data,
                         {"Content-type": type})
            resp = conn.getresponse()
            reply = resp.read()
            print "received %d bytes of type %s" % (len(data), type)
            replytype = resp.getheader("Content-type")
            connection.Process(reply, replytype)
    except Exception as ex:
        print ex
        loop.quit()

def SessionChanged(object, ready):
    print "SessionChanged:", object, ready
    if not ready:
        loop.quit()

bus.add_signal_receiver(Abort,
                        'Abort',
                        'org.syncevolution.Connection',
                        'org.syncevolution',
                        conpath,
                        byte_arrays=True)
bus.add_signal_receiver(Reply,
                        'Reply',
                        'org.syncevolution.Connection',
                        'org.syncevolution',
                        conpath,
                        byte_arrays=True)
bus.add_signal_receiver(SessionChanged,
                        'SessionChanged',
                        'org.syncevolution.Server',
                        'org.syncevolution',
                        None,
                        byte_arrays=True)

# start a test session with an HTTP server
connection.Process(sys.argv[1], "HTTP Config")
loop.run()



#! /usr/bin/python

'''Usage: syncevo-http-server.py <URL>
Runs a SyncML HTTP server under the given base URL.'''

# use the same glib main loop in D-Bus and twisted
from dbus.mainloop.glib import DBusGMainLoop
from twisted.internet import glib2reactor # for non-GUI apps
DBusGMainLoop(set_as_default=True)
glib2reactor.install()

import dbus
import gobject
import sys
import urlparse
import optparse
import os
import atexit
import signal
import time
import subprocess
import logging
import logging.config

import twisted.web
import twisted.python.log
import twisted.web.resource
from twisted.web import server, resource, http
from twisted.internet import ssl, reactor
from OpenSSL import SSL

# for output from this script itself
logger = logging.getLogger("syncevo-http")

# Timeout for D-Bus method calls. Both synchronous and asynchronous
# calls have a low (?) default timeout of 60 seconds. We increase that
# to something which is essentially infinite, to allow more time for
# processing or interactive debugging.
timeout=100000

class ChainedOpenSSLContextFactory(ssl.DefaultOpenSSLContextFactory):
    def __init__(self, privateKeyFileName, certificateChainFileName,
                 sslmethod = SSL.TLSv1_METHOD):
        """
        @param privateKeyFileName: Name of a file containing a private key
        @param certificateChainFileName: Name of a file containing a certificate chain
        @param sslmethod: The SSL method to use
        """
        self.privateKeyFileName = privateKeyFileName
        self.certificateChainFileName = certificateChainFileName
        self.sslmethod = sslmethod
        self.cacheContext()
    
    def cacheContext(self):
        ctx = SSL.Context(self.sslmethod)
        ctx.use_certificate_chain_file(self.certificateChainFileName)
        ctx.use_privatekey_file(self.privateKeyFileName)
        self._context = ctx

# cached information about previous POST and reply,
# in case that we need to resend
class OldRequest:
    sessionid = None
    data = None
    reply = None
    type = None

# holds global variables which will be set later:
# - bus = D-Bus session bus
# - dbusserver = path to syncevo-dbus-server if it needs to be started, None if using system
class Context:
    bus = None
    dbusserver = None
    _server = None

    @staticmethod
    def checkServer():
        if Context._server:
            retcode = Context._server.poll()
            if retcode == None:
                return
            elif retcode < 0:
                logger.error("syncevo-dbus-server was terminated by signal %d", -retcode)
            elif retcode > 0:
                logger.error("syncevo-dbus-server failed, return code %d", retcode)
            else:
                logger.debug("syncevo-dbus-server shut down normally")
            Context._server = None

    @staticmethod
    def getDBusServer():
        if not Context.dbusserver:
            # standard D-Bus activation
            return dbus.Interface(Context.bus.get_object('org.syncevolution',
                                                         '/org/syncevolution/Server'),
                                  'org.syncevolution.Server')

        # check status of previously started server
        Context.checkServer()

        # start executable
        if not Context._server:
            logger.debug("starting %s", Context.dbusserver)
            Context._server = subprocess.Popen(Context.dbusserver, stdout=open("/dev/null", "w"))

        # wait until the daemon shows up on D-Bus (slightly racy)
        while Context._server.poll() == None:
            time.sleep(0.5)
            if 'org.syncevolution' in Context.bus.list_names():
                return dbus.Interface(Context.bus.get_object('org.syncevolution',
                                                             '/org/syncevolution/Server'),
                                      'org.syncevolution.Server')

        # Startup failed?! Clean up, then give up and return nothing (probably triggers further errors).
        Context.checkServer()
        return None

class SyncMLSession:
    sessions = []

    def __init__(self):
        self.sessionid = None
        self.request = None
        self.aborted_request = None
        self.conpath = None
        self.abort_match = None
        self.reply_match = None
        self.connection = None

    def destruct(self, code, message=""):
        '''Tell both HTTP client and D-Bus server that we are shutting down,
        then remove the session'''
        logger.debug("destructing connection %s with code %s message %s", self.conpath, code, message)
        if self.request:
            self.request.setResponseCode(code, message)
            self.request.finish()
            self.request = None
        if self.connection:
            try:
                self.connection.Close(False, message, timeout=timeout)
            except dbus.exceptions.DBusException, ex:
                if ex.get_dbus_name() == "org.freedesktop.DBus.Error.UnknownMethod":
                    # triggered if connection instance is already gone, hide from user
                    logger.debug("self.connection.Close() failed, connection probably already gone: %s", ex)
                else:
                    raise
            self.connection = None
        if self.abort_match:
            Context.bus.remove_signal_receiver(self.abort_match)
        if self.reply_match:
            Context.bus.remove_signal_receiver(self.reply_match)
        if self in SyncMLSession.sessions:
            SyncMLSession.sessions.remove(self)
            logger.debug("removed SyncML session %s", self)

    def abort(self, **keywords):
        '''D-Bus server requests to close connection, so cancel everything'''
        conpath = keywords['conpath']
        logger.debug("connection %s went down, active connection %s", conpath, self.conpath)
        if conpath == self.conpath:
            self.destruct(http.INTERNAL_SERVER_ERROR, "lost connection to SyncEvolution")
        else:
            logger.debug("ignore shutdown of obsolete connection")

    def reply(self, data, type, meta, final, session, **keywords):
        '''sent reply to HTTP client and/or close down normally'''
        conpath = keywords['conpath']
        logger.debug("reply session %s connection %s (active %s) final %s data len %d %s",
                     session, conpath, self.conpath, final, len(data), meta)
        if conpath != self.conpath:
            logger.debug("ignore reply via obsolete connection")
            return
        self.logMessage("outgoing", self.request, data, type)
        # When the D-Bus server sends an empty array, Python binding
        # puts the four chars in 'None' into the data array?!
        if data and len(data) > 0 and data != 'None':
            request = self.request
            self.request = None
            OldRequest.reply = data
            OldRequest.type = type
            if request:
                request.setHeader('Content-Type', type)
                request.setHeader('Content-Length', len(data))
                request.setResponseCode(http.OK)
                request.write(data)
                request.finish()
                self.sessionid = session
            else:
                # syncevo-dbus-server does not need to know about lost connection
                # to client, because client might still resend
                logger.debug("could not send reply immediately, buffering it")
        if final:
            logger.debug("closing connection for connection %s session %s", self.conpath, session)
            if self.connection:
                self.connection.Close(True, "", timeout=timeout)
                self.connection = None
            self.destruct(http.GONE, "D-Bus server done")

    def done(self, error):
        '''lost connection to HTTP client, either normally or in error'''
        logger.debug("done with request in session %s, error %s", self.sessionid, error)
        # Keep request if client closed the connection, the client
        # might be about to resend.
        if error != None:
            self.aborted_request = self.request
        else:
            self.aborted_request = None
        # keep connection to syncevo-dbus-server, client might still
        # retry the request
        self.request = None

    def start(self, request, config, url):
        '''start a new session based on the incoming message'''
        data = request.content.read()
        type = request.getHeader('content-type')
        self.logMessage("incoming", request, data, type)
        logger.debug("requesting new session")
        self.object = Context.getDBusServer()
        self.request = request
        deferred = request.notifyFinish()
        deferred.addCallback(self.done)
        deferred.addErrback(self.done)
        self.conpath = self.object.Connect({'description': 'syncevo-server-http.py',
                                            'transport': 'HTTP',
                                            'config': config,
                                            'URL': url},
                                           True,
                                           '',
                                           timeout=timeout)
        logger.debug("started new connection %s" % self.conpath)
        self.connection = dbus.Interface(Context.bus.get_object('org.syncevolution',
                                                                self.conpath),
                                         'org.syncevolution.Connection')

        if self.abort_match:
            Context.bus.remove_signal_receiver(self.abort_match)
        if self.reply_match:
            Context.bus.remove_signal_receiver(self.reply_match)

        self.abort_match = \
        Context.bus.add_signal_receiver(self.abort,
                                        'Abort',
                                        'org.syncevolution.Connection',
                                        'org.syncevolution',
                                        self.conpath,
                                        path_keyword='conpath',
                                        utf8_strings=True,
                                        byte_arrays=True)
        self.reply_match = \
        Context.bus.add_signal_receiver(self.reply,
                                        'Reply',
                                        'org.syncevolution.Connection',
                                        'org.syncevolution',
                                        self.conpath,
                                        path_keyword='conpath',
                                        utf8_strings=True,
                                        byte_arrays=True)

        # feed new data into SyncEvolution and wait for reply
        request.content.seek(0, 0)
        self.connection.Process(data, type, timeout=timeout)
        SyncMLSession.sessions.append(self)
        logger.debug("added new SyncML session %s", self)

    def process(self, request, data):
        '''process next message by client in running session'''
        type = request.getHeader('content-type')
        self.logMessage("incoming", request, data, type)
        mustprocess = True
        if self.request:
            # Message resend?! Ignore old, active request.
            self.logMessage("new message while still processing one on active TCP connection", self.request, self.request.processingdata, self.request.processingtype)
            self.request.finish()
            if self.request.processingdata == data and self.request.processingtype == type:
                logger.debug("is message resend, no need to process again")
                mustprocess = False
            else:
                logger.debug("different message, send to syncevo-dbus-server")
            self.request = None
        if self.aborted_request != None:
            # Message resend after closing connection and thus aborting the currently
            # active request?
            self.logMessage("new message after aborting the previous one", self.aborted_request, self.aborted_request.processingdata, self.aborted_request.processingtype)
            if self.aborted_request.processingdata == data and self.aborted_request.processingtype == type:
                logger.debug("is message resend, no need to process again")
                mustprocess = False
            else:
                logger.debug("different message, send to syncevo-dbus-server")
            self.aborted_request = None
        deferred = request.notifyFinish()
        deferred.addCallback(self.done)
        deferred.addErrback(self.done)
        self.request = request
        request.processingdata = data
        request.processingtype = type
        if mustprocess:
            self.connection.Process(data, type, timeout=timeout)

    def logMessage(self, direction, request, data, type):
        if 'plain' in type or "+xml" in type:
            logger.debug("processing %s message of type %s and length %d:\n%s" % (direction, type, len(data), data))
        else:
            logger.debug("processing %s message of type %s and length %d, binary data" % (direction, type, len(data)))

class SyncMLPost(resource.Resource):
    isLeaf = True

    def __init__(self, url):
        self.url = url

    def render_GET(self, request):
        logger.info("GET %s from %s", self.url, request.getClientIP())
        return "<html>SyncEvolution SyncML Server</html>"

    def render_POST(self, request):
        config = request.postpath
        if config:
            config = config[0]
        else:
            config = ""
        type = request.getHeader('content-type')
        len = request.getHeader('content-length')
        sessionid = request.args.get('sessionid')
        if sessionid:
            sessionid = sessionid[0]
        logger.debug("POST from %s config %s type %s session %s args %s length %s",
                     request.getClientIP(), config, type, sessionid, request.args, len)
        if not sessionid:
            logger.info("new SyncML session for %s", request.getClientIP())
            session = SyncMLSession()
            session.start(request, config,
                          urlparse.urljoin(self.url.geturl(), request.path))
            return server.NOT_DONE_YET
        else:
            data = request.content.read()
            # Detect resent message. We support that for
            # independently from the session, because it
            # might already be gone (server sends last reply
            # in session, closes session, client doesn't
            # get reply, reposts).
            if sessionid == OldRequest.sessionid and \
                    OldRequest.data == data and \
                    OldRequest.reply:
                logger.debug("resend reply session %s", sessionid)
                request.setHeader('Content-Type', OldRequest.type)
                request.setResponseCode(http.OK)
                request.write(OldRequest.reply)
                request.finish()
                return server.NOT_DONE_YET
            else:
                # prepare resending, will be completed in
                # SyncSession.reply()
                OldRequest.sessionid = sessionid
                OldRequest.data = data
                OldRequest.reply = None
            for session in SyncMLSession.sessions:
                if session.sessionid == sessionid:
                    session.process(request, data)
                    return server.NOT_DONE_YET
            # fallback when session not found
            logger.error("unknown session %s => 404 error", sessionid)
            page = twisted.web.resource.NoResource(message="The session %s was not found" % sessionid)
            return page.render(request)

class TwistedLogging(object):
    "same as Twisted's PythonLoggingObserver, except that it uses loglevels debug and error"
    def __init__(self):
        self.logger = logging.getLogger("twisted")

    def emit(self, eventDict):
        if 'logLevel' in eventDict:
            level = eventDict['logLevel']
        elif eventDict['isError']:
            if 'failure' in eventDict and \
                    eventDict['failure'].type == dbus.exceptions.DBusException and \
                    eventDict['failure'].value.get_dbus_name() == "org.syncevolution.Exception":
                # special case: errors inside the syncevo-dbus-server are better shown
                # to users as part of the syncevo-dbus-server output, so treat the
                # syncevo-http-server side of it as something for debugging.
                level = logging.DEBUG
            else:
                level = logging.ERROR
        else:
            level = logging.DEBUG
        text = twisted.python.log.textFromEventDict(eventDict)
        if text is None:
            return
        self.logger.log(level, text)

    def start(self):
        twisted.python.log.startLoggingWithObserver(self.emit, setStdout=False)

    def stop(self):
        twisted.python.log.removeObserver(self.emit)

evo2python = {
    "DEBUG": logging.DEBUG,
    "DEVELOPER": logging.DEBUG,
    "INFO": logging.INFO,
    "SHOW": logging.INFO,
    "ERROR": logging.ERROR,
    "WARNING": logging.WARNING
}

def logSyncEvoOutput(path, level, output, component):
    logging.getLogger(component or 'sync').log(evo2python.get(level, logging.ERROR), "%s: %s", path, output)

usage =  """usage: %prog [options] http://localhost:<port>/<path>

Runs a HTTP server which listens on all network interfaces on
the given port and answers requests for the given path.
Configurations for clients must be created manually, see
http://syncevolution.org/development/http-server-howto"""

def main():
    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-d", "--debug",
                      action="store_true", dest="debug", default=False,
                      help="enables debug messages")
    parser.add_option("-q", "--quiet",
                      action="store_true", dest="quiet", default=False,
                      help="limits output to real error messages; ignored if --debug is given")
    parser.add_option("", "--log-config",
                      action="store", type="string", dest="logConfig", default=None,
                      help="configure logging via Python logging config file; --debug and --quiet override the log level in the root logger")
    parser.add_option("", "--server-certificate",
                      action="store", type="string", dest="cert", default=None,
                      help="certificate file used by the server to identify itself (required for https)")
    parser.add_option("", "--server-key",
                      action="store", type="string", dest="key", default=None,
                      help="key file used by the server to identify itself (optional, certificate file is used as fallback, which then must contain key and certificate)")
    parser.add_option("", "--start-dbus-session",
                      action="store_true", dest="startDBus", default=False,
                      help="""creates a new D-Bus session for
communication with syncevo-dbus-server and (inside that server) with
other D-Bus services like Evolution Data Server, removes it when
shutting down; should only be used if it is guaranteed that the
current user will not have another session running, because these
services can get confused when started multiple times; without this
option, syncevo-http-server a) uses the session specified in the
environment or b) looks for a session of the current user (depends on
ConsoleKit and might not always work)""")
    parser.add_option("", "--start-syncevolution",
                      action="store_true", dest="startSyncEvo", default=False,
                      help="""sets up the right environment for
syncevo-dbus-server and (re)starts it explicitly, instead of depending
on D-Bus auto-activation; to be used when SyncEvolution is not
installed at the location it was compiled for""")
    parser.add_option("", "--syncevolution-path",
                      action="store", type="string", dest="path", default=None,
                      help="""sets the installation path (the
directory which contains 'bin', 'libexec', etc.) to be used in
--start-syncevolution, default is the location where
syncevo-http-server itself is installed""")
    (options, args) = parser.parse_args()

    # determine level chosen via command line
    level = None
    if options.debug:
        level = logging.DEBUG
    elif options.quiet:
        level = logging.ERROR
    else:
        level = None

    # create logging infrastructure
    if options.logConfig:
        logging.config.fileConfig(options.logConfig)
        # only override level if explicitly set
        if level != None:
            logging.getLogger().setLevel(level)
    else:
        root = logging.getLogger()
        ch = logging.StreamHandler()
        formatter = logging.Formatter("[%(levelname)s" +
                                      (level == logging.DEBUG and " %(asctime)s" or "") +
                                      "] %(name)s: %(message)s")
        ch.setFormatter(formatter)
        root.addHandler(ch)
        if level == None:
            level = logging.INFO
        root.setLevel(level)

    # redirect output from Twisted
    observer = TwistedLogging()
    observer.start()

    # Set up D-Bus. First check for an existing session in env.
    #
    # Doing the check via ConsoleKit here would be nice, but that
    # led to the following problem with --start-dbus-session:
    # - dbus module is initialized in dbus.SystemBus()
    # - DISPLAY remains unset, dbus-launch creates new DBUS_SESSION_BUS_ADDRESS
    # - dbus.SessionBus() fails with "Autolaunch error: X11 initialization failed."
    # Presumably the session bus address is not checked again after setting it.
    # Not touching D-Bus before dbus-launch avoids that problem.
    havedbus = os.environ.get("DBUS_SESSION_BUS_ADDRESS") or os.environ.get("DISPLAY")
    if options.startDBus:
        if havedbus:
            logger.info("%s session found (DISPLAY=%s DBUS_SESSION_BUS_ADDRESS=%s), but starting a new D-Bus session as requested",
                        os.environ.get("DBUS_SESSION_BUS_ADDRESS") and "D-Bus" or "potential",
                        os.environ.get("DISPLAY", ""),
                        os.environ.get("DBUS_SESSION_BUS_ADDRESS", ""))
        p = subprocess.Popen("dbus-launch", stdout=subprocess.PIPE)
        output = p.communicate()[0]
        if p.wait():
            logger.error("dbus-launch failed")
            exit(1)
        for line in output.split():
            logger.debug("dbus-launch: %s", line)
            var, value = line.split("=", 1)
            if var == "DBUS_SESSION_BUS_PID":
                # kill that process when we quit
                atexit.register(os.kill, int(value), signal.SIGTERM)
            os.environ[var] = value
    else:
        if not havedbus:
            # code inspired by Ross Burton's http://burtonini.com/blog/computers/offlineimap-2008-11-04-20-00
            logger.debug("looking for X11 session via ConsoleKit")
            bus = dbus.SystemBus()
            try:
                manager_obj = bus.get_object('org.freedesktop.ConsoleKit', '/org/freedesktop/ConsoleKit/Manager')
                manager = dbus.Interface(manager_obj, 'org.freedesktop.ConsoleKit.Manager')

                for ssid in manager.GetSessionsForUnixUser(os.getuid()):
                    obj = bus.get_object('org.freedesktop.ConsoleKit', ssid)
                    session = dbus.Interface(obj, 'org.freedesktop.ConsoleKit.Session')
                    dpy = session.GetX11Display()
                    logger.debug("ConsoleKit session %s has DISPLAY=%s", session, dpy)
                    if dpy:
                        if havedbus:
                            # already found a session earlier, now what?!
                            logger.error("multiple X11 sessions found for current user, please set DISPLAY to the one to be used for syncing")
                            exit(1)
                        else:
                            # use this X11 session to find D-Bus session bus
                            os.environ["DISPLAY"] = dpy
                            havedbus = True
            except dbus.exceptions.DBusException, ex:
                if ex.get_dbus_name() == "org.freedesktop.DBus.Error.ServiceUnknown":
                    logger.debug("org.freedesktop.ConsoleKit service not available")
                else:
                    raise

        if not havedbus:
            logger.error("no D-Bus session, use --start-dbus-session")
            exit(1)

    # Now at least one of DISPLAY or DBUS_SESSION_BUS_ADDRESS should be set.
    # DISPLAY is sufficient for the D-Bus autolaunch mechanism in libdbus (which
    # reuses a session bus if one exists or creates a new one),
    # whereas DBUS_SESSION_BUS_ADDRESS explicitly selects an existing bus.
    logger.debug("connecting to D-Bus session with DISPLAY=%s DBUS_SESSION_BUS_ADDRESS=%s",
                 os.environ.get("DISPLAY", ""),
                 os.environ.get("DBUS_SESSION_BUS_ADDRESS", ""))
    Context.bus = dbus.SessionBus()

    # catch output from syncevo-dbus-server
    Context.bus.add_signal_receiver(logSyncEvoOutput,
                                    "LogOutput",
                                    "org.syncevolution.Server",
                                    "org.syncevolution",
                                    None)

    # start syncevo-dbus-server?
    if options.startSyncEvo:
        path = options.path
        if not path:
            path = os.path.dirname(os.path.dirname(sys.argv[0]))
        Context.dbusserver = os.path.join(path, "libexec/syncevo-dbus-server")
        logger.info("using SyncEvolution installation in %s", path)
        os.environ["SYNCEVOLUTION_BACKEND_DIR"] = os.path.join(path, "lib/syncevolution/backends")
        os.environ["LD_LIBRARY_PATH"] = ":".join((os.path.join(path, "lib/"),
                                                  os.path.join(path, "lib/syncevolution"),
                                                  os.environ.get("LD_LIBRARY_PATH", "")))
        os.environ["PATH"] = ":".join((os.path.join(path, "libexec/"),
                                       os.path.join(path, "bin"),
                                       os.environ.get("PATH", "")))
        os.environ["SYNCEVOLUTION_DATA_DIR"] = os.path.join(path, "share/syncevolution")
        os.environ["SYNCEVOLUTION_XML_CONFIG_DIR"] = os.path.join(path, "share/syncevolution/xml")
        os.environ["SYNCEVOLUTION_TEMPLATE_DIR"] = os.path.join(path, "share/syncevolution/templates")
        # try whether it can be started
        Context.getDBusServer()

    if len(args) != 1:
        logger.error("need exactly on URL as command line parameter")
        exit(1)

    url = urlparse.urlparse(args[0])
    root = resource.Resource()
    root.putChild(url.path[1:], SyncMLPost(url))
    site = server.Site(root)
    if url.scheme == "https":
        if not options.cert:
            logger.error("need server certificate for https")
            exit(1)
        port = reactor.listenSSL(url.port, site,
                                 ChainedOpenSSLContextFactory(options.key or options.cert, options.cert))
    else:
        port = reactor.listenTCP(url.port, site)
    logger.info("listening on port %d", port.getHost().port)
    reactor.run()

if __name__ == '__main__':
    main()



#! /usr/bin/python

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

DBusGMainLoop(set_as_default=True)

bus = dbus.SessionBus()

object = dbus.Interface(bus.get_object('org.syncevolution',
                                       '/org/syncevolution/Server'),
                        'org.syncevolution.Server')

conpath = object.Connect({'description': 'dbus-server-connection.py',
                          'transport': 'dummy'},
                         False,
                         0)
print conpath

connection = dbus.Interface(bus.get_object('org.syncevolution',
                                           conpath),
                            'org.syncevolution.Connection')
connection.Close(False, 'die, connection, die')


loop = gobject.MainLoop()

conpath = object.Connect({'description': 'dbus-server-connection.py',
                          'transport': 'dummy'},
                         False,
                         0)
connection = dbus.Interface(bus.get_object('org.syncevolution',
                                           conpath),
                            'org.syncevolution.Connection')

def Reply(data, type, meta, final, session):
    print "Reply:", data, type, meta, final, session
    connection.Close(True, '')

sessionpath = None
def SessionChanged(object, ready):
    print "SessionChanged:", object, ready
    if not ready or sessionpath == object:
        loop.quit()

bus.add_signal_receiver(Reply,
                        'Reply',
                        'org.syncevolution.Connection',
                        'org.syncevolution',
                        conpath,
                        byte_arrays=True)
bus.add_signal_receiver(SessionChanged,
                        'SessionChanged',
                        'org.syncevolution.Server',
                        'org.syncevolution',
                        None,
                        byte_arrays=True)
connection.Process([ 1, 2, 3, 4 ], "dummy message type")
loop.run()

sessionpath = object.StartSession('no_such_server')
session = dbus.Interface(bus.get_object('org.syncevolution',
                                        sessionpath),
                         'org.syncevolution.Session')
# wait for session ready
loop.run()
session.Close()
# wait for session gone
loop.run()
